#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import os, datetime, sys, traceback, csv, pickle
import Kross, Plan

T = Kross.module("kdetranslation")

def i18n(text, args = []):
    if T is not None:
        return T.i18n(text, args).decode('utf-8')
    # No translation module, return the untranslated string
    for a in range( len(args) ):
        text = text.replace( ("%" + "%d" % ( a + 1 )), str(args[a]) )
    return text

class BusyinfoExporter:

    def __init__(self, scriptaction):
        self.scriptaction = scriptaction
        self.currentpath = self.scriptaction.currentPath()

        self.proj = Plan.project()
        
        self.forms = Kross.module("forms")
        self.dialog = self.forms.createDialog(i18n("Busy Information Export"))
        self.dialog.setButtons("Ok|Cancel")
        self.dialog.setFaceType("List") #Auto Plain List Tree Tabbed

        datapage = self.dialog.addPage(i18n("Schedules"),i18n("Export Selected Schedule"),"document-export")
        self.scheduleview = Plan.createScheduleListView(datapage)
        
        savepage = self.dialog.addPage(i18n("Save"),i18n("Export Busy Info File"),"document-save")
        self.savewidget = self.forms.createFileWidget(savepage, "kfiledialog:///kplatobusyinfoexportsave")
        self.savewidget.setMode("Saving")
        self.savewidget.setFilter("*.rbi|%(1)s\n*|%(2)s" % { '1' : i18n("Resource Busy Information"), '2' : i18n("All Files") } )

        if self.dialog.exec_loop():
            try:
                self.doExport( self.proj )
            except:
                self.forms.showMessageBox("Error", i18n("Error"), "%s" % "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) ))

    def doExport( self, project ):
        filename = self.savewidget.selectedFile()
        if not filename:
            self.forms.showMessageBox("Sorry", i18n("Error"), i18n("No file selected"))
            return
        schId = self.scheduleview.currentSchedule()
        if schId == -1:
            self.forms.showMessageBox("Sorry", i18n("Error"), i18n("No schedule selected"))
            return
        file = open( filename, 'wb' )
        p = []
        p.append( project.id() )
        p.append( project.data( project, 'NodeName' ) )
        pickle.dump( p, file )
        for i in range( project.resourceGroupCount() ):
            g = project.resourceGroupAt( i )
            for ri in range( g.resourceCount() ):
                r = g.resourceAt( ri )
                lst = r.appointmentIntervals( schId )
                for iv in lst:
                    iv.insert( 0, r.id() )
                    iv.insert( 1, project.data( r, 'ResourceName' ) )
                    pickle.dump( iv, file )

        file.close()

BusyinfoExporter( self )



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import traceback
import Kross
import Plan
import TestResult


TestResult.setResult( True )
asserttext1 = "Test of property '{0}' failed:\n   Expected: '{2}'\n     Result: '{1}'"
asserttext2 = "Failed to set property '{0}' to '{1}'. Result: {2}"

try:
    project = Plan.project()
    assert project is not None, "Project not found"
    
    group1 = project.createResourceGroup()
    assert group1 is not None, "Failed to create resource group"

    property = 'Name'
    data = "G1"
    res = project.setData(group1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(group1, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text

    property = 'Type'
    data = "Material"
    res = project.setData(group1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(group1, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text

    group2 = project.createResourceGroup()
    assert group2 is not None, "Failed to create resource group"

    props = project.resourcePropertyList()
    for p in props:
        data = project.data(group1, p, 'ProgramRole')
        res = project.setData(group2, p, data)
        if res != 'ReadOnly':
            text = asserttext2.format(property, data, res)
            assert res == 'Success', text
    
    
    r1 = project.createResource( group1 )
    assert r1 is not None, "Could not create resource"

    property = 'Name'
    data = "R1"
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Type'
    data = "Material"
    before = project.data(r1, property)
    
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Initials'
    data = "RR"
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Email'
    data = "R1@work.org"
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Email'
    data = "R1@work.org"
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Limit'
    data = 10
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'AvailableFrom'
    data = "2011-07-01T08:00:00"
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'AvailableUntil'
    data = "2011-07-02T08:00:00"
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'NormalRate'
    data = 111
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'OvertimeRate'
    data = 222
    res = project.setData(r1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    account = project.createAccount(0)
    data = 'A1'
    assert account is not None, "Failed to create account"
    res = project.setData(account, 'Name', data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    
    property = 'Account'
    res = project.setData(r1, property, data) 
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    r2 = project.createResource( group2 )
    assert r2 is not None, "Could not create resource"

    for p in props:
        data = project.data(r1, p, 'ProgramRole')
        res = project.setData(r2, p, data)
        if res != 'ReadOnly':
            text = asserttext2.format(property, data, res)
            assert res == 'Success', text

except:
    TestResult.setResult( False )
    TestResult.setMessage("\n" + traceback.format_exc(1))



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import os, datetime, sys, traceback, pickle
import Kross, Plan

T = Kross.module("kdetranslation")

class BusyinfoImporter:

    def __init__(self, scriptaction):
        self.scriptaction = scriptaction
        self.currentpath = self.scriptaction.currentPath()

        self.proj = Plan.project()
        
        self.forms = Kross.module("forms")
        self.dialog = self.forms.createDialog(T.i18n("Busy Information Import"))
        self.dialog.setButtons("Ok|Cancel")
        self.dialog.setFaceType("List") #Auto Plain List Tree Tabbed

        openpage = self.dialog.addPage(T.i18n("Open"), T.i18n("Import Busy Info File"),"document-open")
        self.openwidget = self.forms.createFileWidget(openpage, "kfiledialog:///kplatobusyinfoimportopen")
        self.openwidget.setMode("Opening")
        self.openwidget.setFilter("*.rbi|%(1)s\n*|%(2)s" % { '1' : T.i18n("Resource Busy Information"), '2' : T.i18n("All Files") } )

        if self.dialog.exec_loop():
            try:
                Plan.beginCommand( T.i18nc("(qtundo_format)", "Import resource busy information") )
                self.doImport( self.proj )
                Plan.endCommand()
            except:
                Plan.revertCommand()
                self.forms.showMessageBox("Error", T.i18n("Error"), "%s" % "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) ))

    def doImport( self, project ):
        filename = self.openwidget.selectedFile()
        if not os.path.isfile(filename):
            self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("No file selected") )
            return

        file = open(filename,'r')
        try:
            # load project id and -name
            data = pickle.load( file )
            #print data
            pid = data[0]
            if project.id() == pid:
                self.forms.showMessageBox("Error", T.i18n("Error"), T.i18n("Cannot load data from project with the same identity") )
                raise Exception
            pname = data[1].decode( "UTF-8" )
            # clear existing, so we don't get double up
            project.clearExternalAppointments( pid )
            # load the intervals
            while True:
                data = pickle.load( file )
                self.loadAppointment( project, pid, pname, data )

        except:
            file.close()

    def loadAppointment( self, project, pid, pname, data ):
        r = project.findResource( data[0] )
        if r is None:
            print "Resource is not used in this project: %s, %s" % ( data[0], data[1] )
            return
        if project.data( r, 'ResourceName' ) != data[1]:
            #TODO Warning ?
            print "Resources has same id but different names %s - %s" % ( project.data( r, 'ResourceName' ), data[1] )
        r.addExternalAppointment( pid, pname, data[2:5] )

BusyinfoImporter( self )



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import os, datetime, sys, traceback, pickle
import Kross, Plan

T = Kross.module("kdetranslation")

class ResourcesImporter:

    def __init__(self, scriptaction):
        self.scriptaction = scriptaction

        self.proj = Plan.project()
        
        self.forms = Kross.module("forms")
        self.dialog = self.forms.createDialog(T.i18n("Resources Import"))
        self.dialog.setButtons("Ok|Cancel")
        self.dialog.setFaceType("List") #Auto Plain List Tree Tabbed

        #TODO add options page ( import Calendars? Select calendars, Select resources... )
        
        openpage = self.dialog.addPage(T.i18n("Open"),T.i18n("Plan Resources"),"document-open")
        self.openwidget = self.forms.createFileWidget(openpage, "kfiledialog:///kplatresourcesimportopen")
        self.openwidget.setMode("Opening")
        self.openwidget.setFilter("*.plan|%(1)s\n*|%(2)s" % { '1' : T.i18n("Plan"), '2' : T.i18n("All Files") } )

        if self.dialog.exec_loop():
            try:
                Plan.beginCommand( T.i18nc( "(qtundo_format )", "Import resources" ) )
                self.doImport( self.proj )
                Plan.endCommand()
            except:
                Plan.revertCommand() # play safe in case parts where loaded
                self.forms.showMessageBox("Error", T.i18n("Error"), "%s" % "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) ))

    def doImport( self, project ):
        filename = self.openwidget.selectedFile()
        if not os.path.isfile(filename):
            raise Exception, T.i18n("No file selected")

        Other = Plan.openDocument("Other", filename)
        if Other is None:
            self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("Could not open document: %1", [filename]))
            return
        otherproj = Other.project()
        if otherproj is None:
            self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("No project to import from"))
            return
        if project.id() == otherproj.id():
            self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("Project identities are identical"))
            return

        for ci in range( otherproj.calendarCount() ):
            self.doImportCalendar( project, otherproj.calendarAt( ci ) )
        
        defcal = otherproj.defaultCalendar()
        if defcal is not None:
            dc = project.findCalendar( defcal.id() )
            if dc is not None:
                project.setDefaultCalendar( dc )
        
        for gi in range( otherproj.resourceGroupCount() ):
            othergroup = otherproj.resourceGroupAt( gi )
            gr = project.findResourceGroup( othergroup.id() )
            if gr is None:
                gr = project.createResourceGroup( othergroup )
                if gr is None:
                    self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("Unable to create copy of resource group: %1", [othergroup.name()]))
                    return
            for ri in range( othergroup.resourceCount() ):
                otherresource = othergroup.resourceAt( ri )
                if otherresource is None:
                    self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("No resource to copy from"))
                    return
                self.doImportResource( project, gr, otherresource )

    def doImportResource( self, project, group, resource ):
        r = project.findResource( resource.id() )
        if r is None:
            r = project.createResource( group, resource )
            if r is None:
                self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("Unable to create copy of resource: %1", [resource.name()]))
                return
        else:
            #TODO update?
            print "Resource already exists: %s %s" % ( r.id(), project.data( r, 'ResourceName' ) )
    
    def doImportCalendar( self, project, calendar, parent = None ):
        cal = project.findCalendar( calendar.id() )
        if cal is not None:
            #TODO let user decide
            self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("Calendar already exists: %1", [cal.name()]))
            return
        cal = project.createCalendar( calendar, parent )
        if cal is None:
            self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("Unable to create copy of calendar: %1", [calendar.name()]))
            return
        for ci in range( calendar.childCount() ):
            self.doImportCalendar( project, calendar.childAt( ci ), cal )

ResourcesImporter( self )



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import traceback
import Kross
import Plan
import TestResult


TestResult.setResult( True )
asserttext1 = "Test of property '{0}' failed:\n   Expected: '{2}'\n     Result: '{1}'"
asserttext2 = "Failed to set property '{0}' to '{1}'. Result: {2}"

try:
    project = Plan.project()
    assert project is not None, "Project not found"
    
    task1 = project.createTask( 0 )
    assert task1 is not None, "Could not create task"
    
    task2 = project.createTask( 0 )
    assert task2 is not None, "Could not create task"

    data = "Task name"
    property = 'Name'
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text1
    result = project.data(task1, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    
    data = "Task responsible"
    property = 'Responsible'
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Allocation'
    data = "John Doe,Jane Doe"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'DisplayRole' )
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Estimate'
    data = "3.0d"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'DisplayRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'EstimateType'
    data = "Duration"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'DisplayRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'EstimateType'
    data = "Effort"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'DisplayRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Constraint'
    data = "ALAP"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Constraint'
    data = "ASAP"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Constraint'
    data = "StartNotEarlier"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Constraint'
    data = "MustStartOn"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Constraint'
    data = "FinishNotLater"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Constraint'
    data = "MustFinishOn"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Constraint'
    data = "FixedInterval"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    
    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'ConstraintStart'
    data = "2011-08-01T10:00:00"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'ConstraintEnd'
    data = "2011-08-01T11:00:00"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'OptimisticRatio'
    data = -20
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'PessimisticRatio'
    data = 120
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Constraint'
    data = "ASAP"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    property = 'Estimate'
    data = "4.0w"
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'DisplayRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Risk'
    data = 'Low'
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    property = 'Risk'
    data = 'High'
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Risk'
    data = 'None'
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'StartupCost'
    data = 1000.00
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'ShutdownCost'
    data = 1000.00
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Description'
    data = 'Task description'
    res = project.setData(task1, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task1, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = project.data(task1, property, 'ProgramRole')
    res = project.setData(task2, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

except:
    TestResult.setResult( False )
    TestResult.setMessage("\n" + traceback.format_exc(1))



#! /usr/bin/python -Qwarnall

# This script profiles loading of documents in Calligra.
# It is called like this:
#  profileOfficeFileLoading.py $dir $outputfile
# The script outputs an csv file that contains the times various functions took
# for each file in directory $dir. The directory $dir is scanned recursively.
#
# Copyright 2010 Jos van den Oever <jos@vandenoever.info>

import sys, os, tempfile, time, signal, subprocess, re, lxml.etree, zipfile

applications = {
  'calligrawords': ['odt', 'doc', 'docx'],
  'calligrastage': ['odp', 'ppt', 'pptx'],
  'calligrasheets': ['ods', 'xls', 'xlsx']
}
extensions = {
  'odt': 'odt',
  'doc': 'odt',
  'docx': 'odt',
  'odp': 'odp',
  'ppt': 'odp',
  'pptx': 'odp',
  'ods': 'ods',
  'xls': 'ods',
  'xlsx': 'ods'
}

# limit how many backtraces are recordes, since it takes a lot of time
maxbacktraces = 50

def scanDirectory(rootdir, extensions):
	if os.path.isfile(rootdir):
		return [rootdir]

	filext = map(lambda e: "." + e, extensions)
	list = []
	for root, directories, files in os.walk(rootdir):
		for file in files:
			if file.startswith('.'):
				continue
			if any(map(lambda e: file.endswith(e), filext)):
				list.append(os.path.join(root, file))
	return frozenset(list)

def getExtensions(list):
	extensions = []
	for file in list:
		(path, ext) = os.path.splitext(file)
		extensions.append(ext)
	return frozenset(extensions)

class object:
	def __init__(self):
		pass

# functions for logging, in this case to TeamCity
class logger:
	def __init__(self):
		self.suitename = None
	def escape(self, text):
		def escape(m):
			if m.group(0) == '\n':
				return '|n'
			if m.group(0) == '\r':
				return '|r'
			return '|'+m.group(0)
		return re.sub('[\'\n\r\|\]]', escape, text)
	def startTestSuite(self, name):
		self.suitename = name
		print "##teamcity[testSuiteStarted name='" + self.suitename \
			+ "']"
		sys.stdout.flush()
	def endTestSuite(self):
		if not self.suitename: return
		print "##teamcity[testSuiteFinished name='" + self.suitename \
			+ "']"
		sys.stdout.flush()
		self.suitename = None
	def startTest(self, name):
		if not self.suitename: return
		self.testname = name
		print "##teamcity[testStarted name='" + self.testname + "']"
		sys.stdout.flush()
	# fail the current test
	def failTest(self, backtrace):
		if not self.suitename or not self.testname: return
		bt = ''
		for l in backtrace:
			bt = bt + self.escape(l)
		print "##teamcity[testFailed name='" + self.testname \
			+ "' details='" + bt + "']"
		sys.stdout.flush()
	# end test, pass duration as integer representing the milliseconds
	def endTest(self, duration):
		if not self.suitename or not self.testname: return
		print "##teamcity[testFinished name='" + self.testname \
			+ "' duration='" + str(duration) + "']"
		sys.stdout.flush()
		self.testname = None

def containsRealError(err):
	# some errors reported by libxml2 are false positives, we filter them
	# out
	if str(err).find("ERROR:RELAXNGV:RELAXNG_ERR_CONTENTVALID: Element styles failed to validate content") != -1:
		return None
	if str(err).find("ERROR:RELAXNGV:RELAXNG_ERR_CONTENTVALID: Element automatic-styles failed to validate content") != -1:
		return None
	return err

class odfvalidator:
	def __init__(self):
		path = sys.path[0]
		self.relaxNGValidator = lxml.etree.RelaxNG( \
				lxml.etree.parse(open(os.path.join(path, \
				'OpenDocument-v1.2-cs01-schema-calligra.rng'),
				'r')))
		self.relaxNGManifextValidator = lxml.etree.RelaxNG( \
				lxml.etree.parse(open(os.path.join(path, \
				'OpenDocument-v1.2-cs01-manifest-schema.rng'), \
				'r')))
	# returns error string on error, None otherwise
	def validate(self, odtpath): 
		try:
			zip = zipfile.ZipFile(odtpath, 'r')
		except zipfile.BadZipfile as e:
			return str(e)
		err = self.validateFile(zip, 'content.xml',
				self.relaxNGValidator)
		if (containsRealError(err)):
			return err
		err = self.validateFile(zip, 'styles.xml',
				self.relaxNGValidator)
		if (containsRealError(err)):
			return err
		err = self.validateFile(zip, 'META-INF/manifest.xml',
				self.relaxNGManifextValidator)
		if (containsRealError(err)):
			return err
		return None

	def validateFile(self, zip, file, validator):
		try:
			data = zip.read(file)
			xml = lxml.etree.XML(data);
		except lxml.etree.XMLSyntaxError as e:
			return e
		except KeyError as e:
			return e
		if len(data) > 1000000:
			# if the xml file is larger than 1M, the validator may hang
			return
		if not validator.validate(xml):
			return validator.error_log.last_error

def getExecutablePath(exe):
	exepath = None
	env = os.environ
	for p in env['PATH'].split(':'):
		exepath = os.path.join(p, exe)
		if os.path.exists(exepath):
			break
	return exepath

def runCommand(exepath, arguments, captureStdOut):
	env = os.environ
	stdout = None
	if captureStdOut:
		(fileno, tmpfilename) = tempfile.mkstemp()
		stdout = os.fdopen(fileno, 'w')

	args = 'ulimit -v 1000000 -t 60 -c 0;' + exepath
	for s in arguments:
		args += ' "' + s + '"'

	process = subprocess.Popen(['/bin/bash', '-c', args], env=env,
		close_fds=True, stdout=stdout, stderr=None)
	s = os.wait4(process.pid, os.WNOHANG)
	waited = 0
	waitstep = 0.1
	maxwaittime = 65
	while  s[0] == 0 and s[1] == 0 and waited < maxwaittime:
		# wait a bit
		time.sleep(waitstep)
		waited += waitstep
		s = os.wait4(process.pid, os.WNOHANG)
	if waited >= maxwaittime:
		# if the process did not end nicely, kill it
		try:
			os.killpg(process.pid, signal.SIGKILL)
		except:
			pass
		try:
			s = os.wait4(process.pid, 0)
		except:
			pass
	r = object()
	r.utime = s[2].ru_utime
	r.stime = s[2].ru_stime
	r.returnValue = s[1]
	r.stdout = []
	if captureStdOut:
		stdout.close()
		stdout = open(tmpfilename, 'r')
		r.stdout = stdout.readlines()
		stdout.close()
		os.remove(tmpfilename)
	return r

def profile(dir, file, logger, validator):
	global maxbacktraces
	logger.startTest(file)
	file = os.path.join(dir, file)
	(path, ext) = os.path.splitext(file)
	ext = ext[1:]
	exe = None
	for f in applications.keys():
		if ext in applications[f]:
			exe = f
	exepath = getExecutablePath(exe)

	# profile
	(fileno, tmpfilename) = tempfile.mkstemp()
	roundtripfilename = None
	args = []
	# in case of ODF file, do a roundtrip
	m = re.match('.*(\.od.)$', file)
	(roundtripfd, roundtripfilename) = tempfile.mkstemp("." + extensions[ext])
	if m:
		isOdfFile = True
	else:
		isOdfFile = False
	if isOdfFile:
		args += ["--roundtrip-filename", roundtripfilename]
	args += ["--benchmark-loading", "--profile-filename", tmpfilename,
		"--nocrashhandler", file]
	r = runCommand(exepath, args, False)
	outfile = os.fdopen(fileno, 'r')
	r.lines = outfile.readlines()
	outfile.close()
	os.close(roundtripfd)
	r.backtrace = None
	if r.returnValue != 0:
		if maxbacktraces > 0:
	                maxbacktraces -= 1
			# generate a backtrace
			args = ["--batch", "--eval-command=run",
				"--eval-command=bt", "--args"] + [exepath] + args
			exepath = getExecutablePath("gdb")
			debugresult = runCommand(exepath, args, True)
			r.backtrace = debugresult.stdout
			for l in r.backtrace:
				print l.rstrip()
				sys.stdout.flush()
			logger.failTest(r.backtrace)
		else:
			logger.failTest("Crash, no backtrace: limit reached.")
	else:
		if not isOdfFile:
			# convert ms office file to odf
			exepath = getExecutablePath("calligraconverter")
			args = [file, roundtripfilename]
			cr = runCommand(exepath, args, False)

		err = validator.validate(roundtripfilename);
		if err != None:
			logger.failTest(str(err))

	# remove the roundtripfile and the temporary file
	os.remove(tmpfilename)
	os.remove(roundtripfilename)

	logger.endTest(int((r.utime + r.stime)*1000))
	return r

def getGlobalMinMax(times):
	keys = times.min.keys()
	min = times.min[keys[0]]
	max = times.max[keys[0]]
	for k in keys:
		if times.min[k] <= min:
			min = times.min[k]
		if times.max[k] >= max:
			max = times.max[k]
	return (min, max)

def getGlobalTask(times):
	keys = times.min.keys()
	(min, max) = getGlobalMinMax(times)
	name = None
	for k in keys:
		if times.min[k] == min and times.max[k] == max:
			name = k
	return name

def getFurthestEnd(start, times, excludedkey):
	end = start
	for k in times.min.keys():
		if times.min[k] <= start and times.max[k] > end and k != excludedkey:
			end = times.max[k]
	if end == start:
		end = None
	return end

def getClosestStart(start, times, excludedkey):
	s = 10000000
	for k in times.min.keys():
		if times.min[k] > start and times.min[k] < s and k != excludedkey:
			s = times.min[k]
	return s

# if there is one task that starts before all other tasks and ends after all
# other tasks, then count the time that there is no other task running as a sub-
# task of this global task
# if there is no global task, then count the time between the first and last
# timepoint where there is no task running
def getUnaccountedTime(lines, times):
	if len(times.min.keys()) == 0:
		return 0

	globaltask = getGlobalTask(times)
	(min, max) = getGlobalMinMax(times)
	unaccounted = 0
	currentmax = min
	currentmin = min
	while currentmax != max:
		s = getFurthestEnd(currentmax, times, globaltask)
		if s == None:
			s = getClosestStart(currentmax, times, globaltask)
			if s > max:
				s = max
			unaccounted += s - currentmax
		currentmax = s
	return unaccounted

def summarize(lines):
	r = object()
	r.min = {}
	r.max = {}
	for l in lines:
		(fieldname, time, percentdone) = l.split('\t')
		time = int(time)
		if not fieldname in r.min or r.min[fieldname] > time:
			r.min[fieldname] = time
		if not fieldname in r.max or r.max[fieldname] < time:
			r.max[fieldname] = time
	return r

def profileAll(dir, loggername):
	exts = []
	for v in applications.values():
		for e in v:
			exts.append(e);
	officefiles = scanDirectory(dir, exts);
	usedExts = getExtensions(officefiles)
	results = {}
	log = logger()
	validator = odfvalidator()
	for ext in usedExts:
		if loggername:
			log.startTestSuite(loggername + '-' + ext[1:])
		for f in officefiles:
			(path, pathext) = os.path.splitext(f)
			if pathext == ext:
				relf = os.path.relpath(f, dir)
				result = profile(dir, relf, log, validator)
				results[f] = result
		log.endTestSuite()
	return results

def addMapEntry(map, start, end):
	if start in map:
		a = map[start]
		if end in a:
			a[end] = a[end] + 1
		else:
			a[end] = 1
	else:
		a = {}
		a[end] = 1
	map[start] = a
	return map

def createStackTraceGraph(results):
	exepath = getExecutablePath('dot')
	if not exepath:
		return
	edges = {}
	nodes = {}
	n = 0
	for file in results.keys():
		r = results[file]
		if not r.backtrace:
			continue
		name = os.path.basename(file)
		nodes[name] = file
		for l in r.backtrace:
			l = l.rstrip()
			n += 1
			m = re.search('/calligra/.*/([^/]+:\d+)$', l)
			if m != None:
				key = m.group(1)
				nodes[key] = l
				edges = addMapEntry(edges, key, name)
				name = key

	(fileno, tmpfilename) = tempfile.mkstemp()
	out = os.fdopen(fileno, 'w')
	out.write('digraph {')
	svn = 'http://websvn.kde.org/trunk'
	for a in nodes:
		m = re.search('(/calligra/.*):(\d+)$', nodes[a])
		n = '"' + a + '" [URL = "'
		if m:
			out.write(n + svn + m.group(1) + '?view=markup#l'
				+ m.group(2) + '"];')
		else:
			m = re.search('(/calligratests/.*)', nodes[a])
			if m:
				out.write(n + svn + '/tests' + m.group(1)
					+ '"];')
	for a in edges.keys():
		for b in edges[a].keys():
			out.write('"' + a + '" -> "' + b + '" [penwidth='
				+ str(edges[a][b]) + '];')
	out.write('}')
	os.fdatasync(fileno)
	out.close()

	args = ["-Tsvg", "-ostacktraces.svg", tmpfilename]
	r = runCommand(exepath, args, False)
	os.remove(tmpfilename)

if __name__ == '__main__':
	dir = sys.argv[1]
	output = sys.argv[2]
	loggername = None
	if len(sys.argv) > 3:
		loggername = sys.argv[3]
	results = profileAll(dir, loggername)

	fields = {}
	for r in results.keys():
		results[r].times = summarize(results[r].lines)
		results[r].unaccounted = getUnaccountedTime(results[r].lines,
			results[r].times)
		for f in results[r].times.min:
			if not f in fields:
				fields[f] = object()
				fields[f].count = 0
				fields[f].totalduration = 0
				fields[f].totaldurationsquared = 0
				fields[f].min = 10000000
			min = results[r].times.min[f]
			duration = results[r].times.max[f] - min
			fields[f].totalduration += duration
			fields[f].totaldurationsquared += duration*duration
			fields[f].count += 1
			if fields[f].min > min:
				fields[f].min = min
	fieldnames = fields.keys()
	def sort(keya, keyb):
		return cmp(fields[keya].min, fields[keyb].min)
	fieldnames.sort(sort)

	# collect all fieldnames
	out = open(output, 'wb')
	# write header
	out.write('filename\text\tsize\tr\tutime\tstime\tunaccounted')
	for f in fieldnames:
		out.write('\t' + f)
	out.write('\n')
	# write average
	out.write('average\t\t\t\t\t\t')
	for f in fieldnames:
		f = fields[f]
		out.write('\t')
		if f.count > 0:
			out.write(str(f.totalduration/f.count))
	out.write('\n')
	# write for each analyzed file
	for file in results.keys():
		r = results[file]
		(path, ext) = os.path.splitext(file)
		stat = os.stat(file)
		out.write(file + '\t' + ext[1:] + '\t' + str(stat.st_size) + '\t' + str(r.returnValue))
		out.write('\t' + str(r.utime) + '\t' + str(r.stime))
		out.write('\t' + str(r.unaccounted))
		for f in fieldnames:
			out.write('\t')
			if f in r.times.min:
				out.write(str(r.times.max[f]-r.times.min[f]))
		out.write('\n')
	out.close()

	createStackTraceGraph(results)



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import traceback
import Kross
import Plan
import TestResult


TestResult.setResult( True )
asserttext1 = "Test of property '{0}' failed:\n   Expected: '{2}'\n     Result: '{1}'"
asserttext2 = "Failed to set property '{0}' to '{1}'. Result: {2}"

try:
    project = Plan.project()
    assert project is not None, "Project not found"
    
    group = project.createResourceGroup()
    assert group is not None, "Failed to create resource group"
    
    #Resource group does not have a name() method: should it?
    #property = 'Name'
    #data = "G1"
    #before = group.name()
    #Plan.beginCommand("Set data")
    #res = project.setData(group, property, data)
    #text = asserttext2.format(property, data, res)
    #assert res == 'Success', text
    #result = group.name()
    #text = asserttext1.format(property, result, data)
    #assert result == data, text
    #Plan.revertCommand()
    #result = group.name()
    #text = asserttext1.format(property, result, before)
    #assert result == before, text

    property = 'Name'
    data = "G1"
    before = project.data(group, property)
    Plan.beginCommand("Set data")
    res = project.setData(group, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(group, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(group, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Type'
    data = "Material"
    before = project.data(group, property)
    Plan.beginCommand("Set data")
    res = project.setData(group, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(group, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    
    property = 'Type'
    data = "Work"
    res = project.setData(group, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(group, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(group, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    resource = project.createResource( group )
    assert resource is not None, "Failed to create resource"

    #Should resource have name() method?
    #property = 'Name'
    #data = "R1"
    #before = resource.name()
    #Plan.beginCommand("Set data")
    #res = project.setData(resource, property, data)
    #text = asserttext2.format(property, data, res)
    #assert res == 'Success', text
    #result = resource.name()
    #text = asserttext1.format(property, result, data)
    #assert result == data, text
    #Plan.revertCommand()
    #result = resource.name()
    #text = asserttext1.format(property, result, before)
    #assert result == before, text

    property = 'Name'
    data = "R1"
    before = project.data(resource, property)
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Type'
    data = "Material"
    before = project.data(resource, property)
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = "Work"
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text

    data = "Team"
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text

    Plan.revertCommand()
    result = project.data(resource, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Initials'
    data = "RR"
    before = project.data(resource, property)
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Email'
    data = "R1@work.org"
    before = project.data(resource, property)
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Email'
    data = "R1@work.org"
    before = project.data(resource, property)
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

#        ResourceCalendar,

    property = 'Limit'
    data = 10
    before = project.data(resource, property)
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'AvailableFrom'
    data = "2011-07-01T08:00:00"
    before = project.data(resource, property, 'EditRole')
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property, 'EditRole')
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'AvailableUntil'
    data = "2011-07-02T08:00:00"
    before = project.data(resource, property, 'EditRole')
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property, 'EditRole')
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'NormalRate'
    data = 111
    before = project.data(resource, property, 'EditRole')
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property, 'EditRole')
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'OvertimeRate'
    data = 222
    before = project.data(resource, property, 'EditRole')
    Plan.beginCommand("Set data")
    res = project.setData(resource, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(resource, property, 'EditRole')
    text = asserttext1.format(property, result, before)

    account = project.createAccount(0)
    data = 'A1'
    assert account is not None, "Failed to create account"
    res = project.setData(account, 'Name', data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(account, 'Name')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    
    property = 'Account'
    res = project.setData(resource, property, data) 
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(resource, property, 'ProgramRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

except:
    TestResult.setResult( False )
    TestResult.setMessage("\n" + traceback.format_exc(1))



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import traceback
import Kross
import Plan
import TestResult


TestResult.setResult( True )
asserttext = "Test of property '{0}' failed:\n   Expected: '{2}'\n     Result: '{1}'"
asserttext2 = "Failed to set property '{0}' to '{1}'. Result: {2}"

try:
    project = Plan.project()
    assert project is not None
    
    calendar = project.createCalendar( 0 )
    assert calendar is not None, "Could not create calendar"

    property = 'Name'
    data = "Calendar name"
    before = calendar.name()
    Plan.beginCommand("Set data")
    res = project.setData(calendar, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = calendar.name()
    text = asserttext.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = calendar.name()
    text = asserttext.format(property, result, before)
    assert result == before, text

    property = 'Name'
    data = "Calendar name"
    before = project.data(calendar, property)
    Plan.beginCommand("Set data")
    res = project.setData(calendar, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(calendar, property)
    text = asserttext.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(calendar, property)
    text = asserttext.format(property, result, before)
    assert result == before, text
    
    property = 'Name'
    data = 2 # Checked
    before = project.data(calendar, property)
    Plan.beginCommand("Set data")
    res = project.setData(calendar, property, data, 'CheckStateRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text + " role = CheckStateRole"
    result = project.data(calendar, property, 'CheckStateRole', -1)
    text = asserttext.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(calendar, property)
    text = asserttext.format(property, result, before)
    assert result == before, text

    property = 'Weekday'
    data = ['Monday', "08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    data = ['Tuesday', "08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    data = ['Wednesday', "08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    data = ['Thursday', "08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    data = ['Friday', "08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    data = ['Saturday', "08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    data = ['Sunday', "08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    data = ['Sunday', 'NonWorking']
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    data = ['Sunday', 'Undefined']
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Date'
    data = ["2011-08-21", "08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    property = 'Noday'
    data = ["08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res != 'Success', text

    property = 'Date'
    data = ["08:30:00", "12:00:00"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res != 'Success', text

    data = ["2011-01-01", "NonWorking"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

    data = ["2011-01-01", "Undefined"]
    res = project.setData(calendar, property, data, 'EditRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text

except:
    TestResult.setResult( False )
    TestResult.setMessage("\n" + traceback.format_exc(1))



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import traceback
import Kross
import Plan
import TestResult


TestResult.setResult( True )
asserttext1 = "Test of property '{0}' failed:\n   Expected: '{2}'\n     Result: '{1}'"
asserttext2 = "Failed to set property '{0}' to '{1}'. Result: {2}"

try:
    project = Plan.project()
    assert project is not None
    
    calendar = project.createCalendar( 0 )
    assert calendar is not None, "Could not create calendar"

    property = 'Name'
    data = "Calendar name"
    res = project.setData(calendar, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    
    property = 'Name'
    data = 2 # Checked
    res = project.setData(calendar, property, data, 'CheckStateRole')
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text + " role = CheckStateRole"

    c2 = project.createCalendar( 0 )
    assert c2 is not None, "Could not create calendar"

    props = project.calendarPropertyList()
    for p in props:
        data = project.data(calendar, p, 'ProgramRole')
        res = project.setData(c2, p, data)
        if res != 'ReadOnly':
            text = asserttext2.format(property, data, res)
            assert res == 'Success', text
        
except:
    TestResult.setResult( False )
    TestResult.setMessage("\n" + traceback.format_exc(1))



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

"""
Python script to import task data from a spreadsheet

(C)2011 Dag Andersen <danders@get2net.dk>
Licensed under LGPL v2.1+higher.

Requires Calligra Sheets

The data in the spreadsheet must be organized as follows:

1) The data must be found in sheet 1
2) Row 1 must define the header text for each column
3) The header text must match the task data property name
4) Column A must contain the WBS Code
5) Column A *MUST* be set to text format
6) The wbs code must be in the format defined by Tools -> Define WBS Pattern
   (Default is arabic numbers with '.' separator, e.g. 1.2.3)
7) The separator must be a '.' for all levels
8) The tasks must be ordered in increasing wbs code order
9) The WBS codes must be consecutive:
   If you have task 1.2 and 1.4 you must also have tasks 1, 1.1 and 1.3

Example spreadsheet:

WBSCode Name    Responsible Allocation  EstimateType    Estimate    Description
1       A       Elvis                                               Summary task
1.2     B       Elvis       Jane        Effort          3d          Task 
1.1     C       Elvis       John        Duration        4d          Task

2       D       Gardner                                             Summary task
2.1     E       Gardner                                             Summary task
2.1.1   F       Gardner     Jane,John   Effort          3h          Next level
2.1.2   G       Gardner                 Effort          0h          Milestone

(Note that the resources listed in Allocation will be created if they do not already exists)
"""

import os, sys, traceback
import Kross, Plan

T = Kross.module("kdetranslation")

class TaskImporter:

    def __init__(self, scriptaction):
        self.scriptaction = scriptaction

        self.proj = Plan.project()
        
        self.forms = Kross.module("forms")
        self.dialog = self.forms.createDialog(T.i18n("Task Import"))
        self.dialog.setButtons("Ok|Cancel")
        self.dialog.setFaceType("List") #Auto Plain List Tree Tabbed
        
        openpage = self.dialog.addPage(T.i18n("Open"),T.i18n("Tables Tasks"),"document-open")
        self.openwidget = self.forms.createFileWidget(openpage, "kfiledialog:///tablestaskimportopen")
        self.openwidget.setMode("Opening")
        self.openwidget.setFilter("*.ods|%(1)s\n*|%(2)s" % { '1' : T.i18n("Tables"), '2' : T.i18n("All Files") } )

        if self.dialog.exec_loop():
            try:
                Plan.beginCommand( T.i18nc( "(qtundoformat)", "Import tasks" ) )
                self.doImport( self.proj )
                Plan.endCommand()
            except:
                Plan.revertCommand() # in case partly loaded
                self.forms.showMessageBox("Error", T.i18n("Error"), "%s" % "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) ))

    def doImport( self, project ):
        filename = self.openwidget.selectedFile()
        if not os.path.isfile(filename):
            raise Exception, T.i18n("No file selected")

        Tables = Kross.module("kspread")
        if not Tables:
            raise Exception, T.i18n("Could not start Calligra Sheets")
        
        if not Tables.openUrl(filename):
            self.forms.showMessageBox("Sorry", T.i18n("Error"), T.i18n("Could not open document: %1", [filename]))
            return

        def findParentTask(parent, wbs):
            if wbs == project.data(parent, 'WBSCode'):
                return parent
            for i in range(parent.childCount()):
                task = findParentTask(parent.childAt(i), wbs)
                if task is not None:
                    return task
            return None
        
        def parentTask(wbs):
            codes = wbs.split('.')
            if len(codes) > 1:
                codes.pop()
                pwbs = '.'.join(codes)
                for i in range(project.taskCount()):
                    parent = findParentTask(project.taskAt(i), pwbs)
                    if parent is not None:
                        return parent
            return project
        
        def createTask(properties, data):
            if len(data) == 0:
                return
            task = project.createTask(parentTask(data[0]))
            if task is None:
                raise Exception, T.i18n("Failed to create task")
            
            for i in range(1, len(properties)):
                project.setData(task, properties[i], data[i])
        
        # Get data into list that can be sorted after wbs code
        reader = Tables.reader()
        props = []
        def changedRow(row):
            if row == 1:
                props.append(reader.currentValues())
            else:
                data = reader.currentValues()
                # skip lines without wbs code
                if data[0] is not None:
                    createTask(props[0], data)

        reader.setSheet("Sheet1")
        reader.connect("changedRow(int)", changedRow)
        reader.start()


TaskImporter( self )



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import os, datetime, sys, traceback, csv, pickle
import Kross, Plan


class BusyinfoCheck:

    def __init__(self, scriptaction):
        proj = Plan.project()
        self.check( proj)
        
    def check(self, project ):
        lst = []
        for i in range( project.resourceGroupCount() ):
            g = project.resourceGroupAt( i )
            for ri in range( g.resourceCount() ):
                r = g.resourceAt( ri )
                lst = r.externalAppointments()
                for iv in lst:
                    iv.insert( 0, r.id() )
                    iv.insert( 1, project.data( r, 'ResourceName' ) )
                print lst

BusyinfoCheck( self )



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import os, datetime, sys, traceback, pickle
import Kross, Plan

T = Kross.module("kdetranslation")

#TODO some ui
Plan.beginCommand( T.i18nc( "(qtundoformat)", "Clear all external appointments" ) )
Plan.project().clearExternalAppointments()
Plan.endCommand()



#!/usr/bin/env python

import sys, os, tempfile, subprocess, lxml.etree, zipfile, urllib, hashlib

def getODFVersion(zip):
	content = lxml.etree.parse(zip.open("content.xml", "r"))
	return content.getroot().get(
		"{urn:oasis:names:tc:opendocument:xmlns:office:1.0}version")

def getJing():
	jingjar = "jing-20091111/bin/jing.jar"
	path = os.path.join(sys.path[0], jingjar)
	if os.path.isfile(path):
		return path
	print "Downloading jing.jar"
	z = "jing-20091111.zip"
	urllib.urlretrieve("http://jing-trang.googlecode.com/files/" + z, z);
	zip = zipfile.ZipFile(z, "r");
	zip.extract(jingjar, sys.path[0])
	os.unlink(z)
	f = open(path, "rb")
	h = hashlib.sha1()
	h.update(f.read())
	f.close()
	hash = h.hexdigest()
	if hash != "daa0cf7b1679264f8e68171f7f253255794773f7":
		print "Wrong hash code: wrong file."
		os.unlink(path)
		return
	return path

schemas = {
	"1.0": ["OpenDocument-schema-v1.0-os.rng",
		"OpenDocument-manifest-schema-v1.0-os.rng"],
	"1.1": ["OpenDocument-schema-v1.1.rng",
		"OpenDocument-manifest-schema-v1.1.rng"],
	"1.2": ["OpenDocument-v1.2-cs01-schema-calligra.rng",
		"OpenDocument-v1.2-cs01-manifest-schema.rng"]
}

def getScriptPath():
	return os.path.dirname(os.path.realpath(sys.argv[0]))

class jingodfvalidator:
	def __init__(self, jingjar):
		self.jingjar = jingjar;
		self.xmlparser = lxml.etree.XMLParser()
		xsltpath = os.path.join(getScriptPath(), "removeForeign.xsl")
		self.removeForeignXSLT = self.loadXSLT(xsltpath)

	def validate(self, odfpath):
		try:
			zip = zipfile.ZipFile(odfpath, 'r')
		except:
			self.validateFlatXML(odfpath)
			return
		odfversion = getODFVersion(zip)
		if not odfversion in schemas:
			return "Document has no version number"
		err = self.validateFile(zip, 'content.xml',
				schemas[odfversion][0])
		if (err):
			return err
		err = self.validateFile(zip, 'styles.xml',
				schemas[odfversion][0])
		if (err):
			return err
		err = self.validateFile(zip, 'META-INF/manifest.xml',
				schemas[odfversion][1])
		if (err):
			return err
		err = self.validateFile(zip, 'meta.xml',
				schemas[odfversion][0])
		if (err):
			return err
		err = self.validateFile(zip, 'settings.xml',
				schemas[odfversion][0])
		if (err):
			return err
		return None

	def validateFlatXML(self, filepath):
		schema = schemas["1.2"][0]
		schema = os.path.join(sys.path[0], schema)
		r = self.validateXML(schema, filepath)
		if r:
			return filepath + " is not valid."

	def validateFile(self, zip, filepath, schema):
		schema = os.path.join(sys.path[0], schema)
		suffix = "_" + filepath.replace("/", "_")
		tmp = tempfile.NamedTemporaryFile(suffix = suffix)
		tmp.write(zip.open(filepath, "r").read())
		tmp.flush()
		r = self.validateXML(schema, tmp.name)
		tmp.close()
		if r:
			return filepath + " is not valid."

	def loadXML(self, filepath):
		return lxml.etree.parse(open(filepath, 'r'), self.xmlparser)

	def loadXSLT(self, filepath):
		xsl = self.loadXML(filepath)
		ac = lxml.etree.XSLTAccessControl(read_network=False, write_file=False)
		return lxml.etree.XSLT(xsl, access_control=ac)

	def removeForeign(self, filepath):
		xml = self.loadXML(filepath)
		xml = self.removeForeignXSLT(xml)
		xml.write(filepath)

	# Validate the XML and optionally remove the foreign elements and attributes
	# first. Calligra currently write ODF 1.2 Extended which is allowed to
	# contain foreign elements and attributes. If Calligra adds a mode to save
	# ODF 1.2, the validator should not remove them when validation.
	def validateXML(self, schema, xmlpath, removeForeign = True):
		if removeForeign:
			self.removeForeign(xmlpath)

		args = ["java", "-jar", self.jingjar, "-i", schema, xmlpath]
		return subprocess.call(args)

def createValidator(name):
	xml = lxml.etree.parse(open(os.path.join(sys.path[0], name), "rb"))
	return lxml.etree.RelaxNG(xml)

class odfvalidator:
	def __init__(self):
		path = sys.path[0]
		self.validators = {}
		for key in schemas.keys():
			self.validators[key] = [
				createValidator(schemas[key][0]),
				createValidator(schemas[key][1])
			]
	# returns error string on error, None otherwise
	def validate(self, odfpath): 
		zip = zipfile.ZipFile(odfpath, 'r')
		odfversion = getODFVersion(zip)
		if not odfversion in schemas:
			return "Document has no version number"
		err = self.validateFile(zip, 'content.xml',
				self.validators[odfversion][0])
		if (err):
			return err
		err = self.validateFile(zip, 'styles.xml',
				self.validators[odfversion][0])
		if (err):
			return err
		err = self.validateFile(zip, 'META-INF/manifest.xml',
				self.validators[odfversion][1])
		if (err):
			return err
		return None

	def validateFile(self, zip, file, validator):
		try:
			xml = lxml.etree.XML(zip.read(file));
		except lxml.etree.XMLSyntaxError as e:
			return file + ':' + str(e)
		except KeyError as e:
			return e
		if not validator.validate(xml):
			return file + ':' + str(validator.error_log.last_error)

if __name__ == '__main__':
	jingjar = getJing()
	if jingjar:
		validator = jingodfvalidator(jingjar)
	else:
		validator = odfvalidator()
	for f in sys.argv[1:]:
		if os.path.isfile(f):
			e = validator.validate(f)
			if e:
				print str(e)



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import os, sys, traceback, tempfile, zipfile
import Kross
import Plan

State_Started = 0
State_StartedLate = 1
State_StartedEarly = 2
State_Finished = 3
State_FinishedLate = 4
State_FinishedEarly = 5
State_Running = 6
State_RunningLate = 7
State_RunningEarly = 8
State_ReadyToStart = 9        # all precceeding tasks finished (if any)
State_NotReadyToStart = 10    # all precceeding tasks not finished (must be one or more)
State_NotScheduled = 11

def testBit(int_type, offset):
    mask = 1 << offset
    return ( int_type & mask ) > 0

def state( int_type ):
    if testBit( int_type, State_NotScheduled ) is True:
        return "Not scheduled"
    if testBit( int_type, State_FinishedEarly ) is True:
        return "Finished early"
    if testBit( int_type, State_FinishedLate ) is True:
        return "Finished late"
    if testBit( int_type, State_Finished ) is True:
        return "Finished"
    if testBit( int_type, State_ReadyToStart ) is True:
        return "Ready to start"
    if testBit( int_type, State_NotReadyToStart ) is True:
        return "Not ready to start"
    if testBit( int_type, State_StartedLate ) is True:
        return "Started late"
    if testBit( int_type, State_StartedEarly ) is True:
        return "Started early"
    if testBit( int_type, State_RunningEarly ) is True:
        return "Running early"
    if testBit( int_type, State_RunningLate ) is True:
        return "Running late"
    if testBit( int_type, State_Running ) is True:
        return "Running"
    if int_type > 0:
        return "Error: Invalid state %d" % ( int_type )
    return "None"

def printStates( node, schedule ):
    printState( node, schedule )
    for i in range( node.childCount() ):
        printStates( node.childAt( i ), schedule )

def printState( node, schedule ):
    if node.type() in [ 'Task' ]:
        st = Plan.project().data( node, 'Status', 'EditRole', schedule )
        print "%-30s %-20s %20s" % ( 
            Plan.project().data( node, 'Name', 'DisplayRole', schedule ),
            Plan.project().data( node, 'Status', 'DisplayRole', schedule ),
            state( int( st ) ) )

def printNodes( node, props, schedule, types = None ):
    printNode( node, props, schedule, types )
    for i in range( node.childCount() ):
        printNodes( node.childAt( i ), props, schedule, types )

def printNode( node, props, schedule, types = None ):
    if types is None or node.type() in types:
        for prop in props:
            print "%-25s" % ( Plan.project().data( node, prop[0], prop[1], schedule ) ),
        print

def printGroup( group, props ):
    for prop in props:
        print "%-25s" % ( Plan.project().data( group, prop ) ),
    print
    for i in range( group.resourceCount() ):
        printResource( group.resourceAt( i ), props )

def printResource( resource, props ):
    for prop in props:
        print "%-25s" % ( Plan.project().data( resource, prop ) ),
    print

def printSchedules():
    print "%-10s %-25s %-10s" % ( "Identity", "Name", "Scheduled" )
    for i in range( proj.scheduleCount() ):
        printSchedule( proj.scheduleAt( i ) )
    print


def printSchedule( sch ):
    print "%-10s %-25s %-10s" % ( sch.id(), sch.name(), sch.isScheduled() )
    for i in range( sch.childCount() ):
        printSchedule( sch.childAt( i ) )

def printEffortCost( name, values ):
    print "%-20s" % ( name )
    for d, v in sorted( values.iteritems() ):
        e = v[0]
        c = v[1]
        print "%-20s %-15s %20f %20f" % ( "", d, e, c )

def printProjectBusyinfo( proj ):
    print "%-20s %-30s %-30s %8s" % ( "Resource", "Start", "End", "Load" )
    id = proj.scheduleAt( 0 ).id()
    for gi in range( proj.resourceGroupCount() ):
        g = proj.resourceGroupAt( gi )
        for ri in range( g.resourceCount() ):
            r = g.resourceAt( ri )
            printBusyinfo( r, r.appointmentIntervals( id ) )
            print
        
    print

def printBusyinfo( res, lst ):
    name = Plan.project().data( res, 'Name' )
    for interval in lst:
        print "%-20s %-30s %-30s %8s" % ( name, interval[0], interval[1], interval[2] )
        name = ""

def printProjectCalendars( proj ):
    for c in range( proj.calendarCount() ):
        printChildCalendars( proj.calendarAt ( c ) )

def printChildCalendars( calendar ):
    print calendar.name()
    for c in range( calendar.childCount() ):
        printChildCalendars( calendar.childAt ( c ) )


def printExternalProjects( proj ):
    projects = proj.externalProjects()
    if len(projects) == 0:
        print "No external project appointments"
        return
    if len(projects) % 2 == 1:
        print "Illegal id/name pair in list: %s" % projects
        return

    print "%-35s %s" % ( "Identity", "Name" )
    for c in projects:
        print "%-35s %s" % ( c[0], c[1] )

def printTaskEffortCost( parent ):
    for i in range( parent.childCount() ):
        node = parent.childAt( i )
        name = Plan.project().data( node, 'Name' )
        printEffortCost( name, node.plannedEffortCostPrDay( "2007-09-12", "2007-09-18", sid ) )

#------------------------
proj = Plan.project()

sid = -1;
# get a schedule id
if proj.scheduleCount() > 0:
    sid = proj.scheduleAt( 0 ).id()

print "Using schedule id: %-3s" % ( sid )
print

nodeprops = [['WBSCode', 'DisplayRole'], ['Name', 'DisplayRole'], ['Type', 'DisplayRole'], ['Responsible', 'DisplayRole'], ['Status', 'EditRole'] ]
print "Print tasks and milestones in arbitrary order:"
# print the localized headers
for prop in nodeprops:
    print "%-25s" % (proj.taskHeaderData( prop ) ),
print
printNodes( proj, nodeprops, sid, [ 'Task', 'Milestone' ] )
print

print "Print all nodes including project:"
for prop in nodeprops:
    print "%-25s" % (proj.taskHeaderData( prop ) ),
print
printNodes( proj, nodeprops, sid )
print

print "Print Resources:"
resprops = [ 'Name', 'Type', 'Email', 'Calendar' ]
# print the localized headers
for prop in resprops:
    print "%-25s" % (proj.resourceHeaderData( prop ) ),
print
for index in range( proj.resourceGroupCount() ):
    g = proj.resourceGroupAt( index )
    printGroup( g, resprops )

print

print "Print Schedules:"
printSchedules()
print


print "Print Effort/Cost for each task:"
print "%-20s %-10s %-10s %-10s" % ( 'Name', 'Date', 'Effort', 'Cost' )
printTaskEffortCost( proj )
print
print "Print Effort/Cost for the project:"
print "%-20s %-10s %-10s %-10s" % ( 'Name', 'Date', 'Effort', 'Cost' )
printEffortCost( proj.name(), proj.plannedEffortCostPrDay( "2007-09-12", "2007-09-17", sid ) )
print
print "Print Busy information for all resources in the project:"
printProjectBusyinfo( proj )
print

print "Print the calendars in the project:"
printProjectCalendars( proj )
print

print "Print planned Effort/Cost for each account:"
print "%-20s %-15s %20s %20s" % ( 'Name', 'Date', 'Effort', 'Cost' )
for i in range( proj.accountCount() ):
    account = proj.accountAt( i )
    name = Plan.project().data( account, 'Name' )
    printEffortCost( name, account.plannedEffortCostPrDay( sid ) )
print

print "Print actual Effort/Cost for each account:"
print "%-20s %-15s %20s %20s" % ( 'Name', 'Date', 'Effort', 'Cost' )
for i in range( proj.accountCount() ):
    account = proj.accountAt( i )
    name = Plan.project().data( account, 'Name' )
    printEffortCost( name, account.actualEffortCostPrDay( sid ) )
print

print "Print BCWS for the project:"
print "%-20s %-15s %20s %20s" % ( 'Name', 'Date', 'Effort', 'Cost' )
printEffortCost( proj.name(), proj.bcwsPrDay( sid ) )
print
print "Print BCWP for the project:"
print "%-20s %-15s %20s %20s" % ( 'Name', 'Date', 'Effort', 'Cost' )
printEffortCost( proj.name(), proj.bcwpPrDay( sid ) )
print

print "Print ACWP for the project:"
print "%-20s %-15s %20s %20s" % ( 'Name', 'Date', 'Effort', 'Cost' )
printEffortCost( proj.name(), proj.acwpPrDay( sid ) )
print

print "Print Task status:"
printStates( proj, sid )
print

print "Print external projects:"
printExternalProjects( proj )
print



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

import traceback
import Kross
import Plan
import TestResult


TestResult.setResult( True )
asserttext1 = "Test of property '{0}' failed:\n   Expected: '{2}'\n     Result: '{1}'"
asserttext2 = "Failed to set property '{0}' to '{1}'. Result: {2}"

try:
    project = Plan.project()
    assert project is not None, "Project not found"
    
    task = project.createTask( 0 )
    assert task is not None, "Could not create task"
    
    data = "Task name"
    property = 'Name'
    before = task.name()
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = task.name()
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = task.name()
    text = asserttext1.format(property, result, before)
    assert result == before, text

    data = "Task name"
    property = 'Name'
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    data = "Task responsible"
    property = 'Responsible'
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property)
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Allocation'
    data = "John Doe"
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'DisplayRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Estimate'
    data = "3.0h"
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'DisplayRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'EstimateType'
    data = "Duration"
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'DisplayRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Constraint'
    data = "ALAP"
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Constraint'
    data = "FixedInterval"
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    
    property = 'ConstraintStart'
    data = "2011-08-01T10:00:00"
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'ConstraintEnd'
    data = "2011-08-01T11:00:00"
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Constraint'
    data = "ASAP"
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    property = 'Type'
    data = 'Task' # Some of the following tests fail if task is something else than a Task
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    property = 'OptimisticRatio'
    data = -20
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'PessimisticRatio'
    data = 120
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Constraint'
    data = "ASAP"
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    property = 'Estimate'
    data = "3.0d"
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'DisplayRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text

    property = 'Risk'
    data = 'Low'
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'StartupCost'
    data = 1000.00
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'ShutdownCost'
    data = 1000.00
    before = project.data(task, property, 'EditRole')
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, before)
    assert result == before, text

    property = 'Description'
    data = 'Task description'
    before = project.data(task, property)
    Plan.beginCommand("Set data")
    res = project.setData(task, property, data)
    text = asserttext2.format(property, data, res)
    assert res == 'Success', text
    result = project.data(task, property, 'EditRole')
    text = asserttext1.format(property, result, data)
    assert result == data, text
    Plan.revertCommand()
    result = project.data(task, property)
    text = asserttext1.format(property, result, before)
    assert result == before, text

except:
    TestResult.setResult( False )
    TestResult.setMessage("\n" + traceback.format_exc(1))



#!/usr/bin/env kross
# -*- coding: utf-8 -*-

"""
Python script to import content from a Plan Project stored
within a Plan file into Tables.

(C)2008 Dag Andersen <danders@get2net.dk>
(C)2011 Dag Andersen <danders@get2net.dk>
Licensed under LGPL v2+higher.
"""

import sys, os, traceback
import Kross, KSpread

T = Kross.module("kdetranslation")

class PlanImport:

    def __init__(self, scriptaction):
        self.scriptaction = scriptaction
        self.currentpath = self.scriptaction.currentPath()
        self.forms = Kross.module("forms")
        try:
            self.start()
        except Exception, inst:
            self.forms.showMessageBox("Sorry", T.i18n("Error"), "%s" % inst)
        except:
            self.forms.showMessageBox("Error", T.i18n("Error"), "%s" % "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) ))

    def start(self):
        writer = KSpread.writer()
        filename = self.showImportDialog(writer)
        if not filename:
            return # no exception, user prob pressed cancel

        Plan = Kross.module("Plan")
        if Plan is None:
            raise Exception, T.i18n("Failed to start Plan. Is Plan installed?")

        Plan.openUrl( filename )
        proj = Plan.project()
        data = self.showDataSelectionDialog( writer, Plan )
        if len(data) == 0:
            raise Exception, T.i18n("No data to import")

        objectType = data[0]
        schedule = data[1]
        props = data[2]
        if len(props) == 0:
            raise Exception, T.i18n("No properties to import")

        record = []
        if data[3] == True:
            for prop in props:
                record.append( proj.headerData( objectType, prop ) )
            if not writer.setValues(record):
                if self.forms.showMessageBox("WarningContinueCancel", T.i18n("Warning"), T.i18n("Failed to set all properties of '%1' to cell '%2'", [", ".join(record), writer.cell()])) == "Cancel":
                    return
            writer.next()

        if objectType == 0: # Nodes
            self.importValues( writer, proj, proj, props, schedule )
        if objectType == 1: # Resources
            for i in range( proj.resourceGroupCount() ):
                self.importValues( writer, proj, proj.resourceGroupAt( i ), props, schedule )
        if objectType == 2: # Accounts
            for i in range( proj.accountCount() ):
                self.importValues( writer, proj, proj.accountAt( i ), props, schedule )

    def importValues(self, writer, project, dataobject, props, schedule ):
        record = []
        for prop in props:
            record.append( project.data( dataobject, prop, "DisplayRole", schedule ) )
        if not writer.setValues(record):
            if self.forms.showMessageBox("WarningContinueCancel", T.i18n("Warning"), T.i18n("Failed to set all properties of '%1' to cell '%2'", [", ".join(record), writer.cell()])) == "Cancel":
                return
        writer.next()
        for i in range( dataobject.childCount() ):
            self.importValues( writer, project, dataobject.childAt( i ), props, schedule )

    def showImportDialog(self, writer):
        dialog = self.forms.createDialog(T.i18n("Plan Import"))
        dialog.setButtons("Ok|Cancel")
        dialog.setFaceType("List") #Auto Plain List Tree Tabbed

        openpage = dialog.addPage(T.i18n("Open"),T.i18n("Import from Plan Project File"),"document-import")
        openwidget = self.forms.createFileWidget(openpage, "kfiledialog:///kspreadplanimportopen")
        openwidget.setMode("Opening")
        openwidget.setFilter("*.plan|%(1)s\n*|%(2)s" % { '1' : T.i18n("Plan Project Files"), '2' : T.i18n("All Files") })
        if dialog.exec_loop():
            filename = openwidget.selectedFile()
            if not os.path.isfile(filename):
                raise Exception, T.i18n("No file selected.")
            return filename
        return None

    def showDataSelectionDialog(self, writer, Plan ):
        tabledialog = self.forms.createDialog("Property List")
        tabledialog.setButtons("Ok|Cancel")
        tabledialog.setFaceType("List") #Auto Plain List Tree Tabbed
        
        datapage = tabledialog.addPage(T.i18n("Destination"),T.i18n("Import to sheet beginning at cell"))
        sheetslistview = KSpread.createSheetsListView(datapage)
        sheetslistview.setEditorType("Cell")

        schedulepage = tabledialog.addPage(T.i18n("Schedules"),T.i18n("Select schedule"))
        schedulewidget = Plan.createScheduleListView(schedulepage)

        sourcepage = tabledialog.addPage(T.i18n("Data"),T.i18n("Select data"))
        sourcewidget = Plan.createDataQueryView(sourcepage)
        if tabledialog.exec_loop():
            currentSheet = sheetslistview.sheet()
            if not currentSheet:
                raise Exception, T.i18n("No current sheet.")
            if not writer.setSheet(currentSheet):
                raise Exception, T.i18n("Invalid sheet '%1' defined.", [currentSheet])

            cell = sheetslistview.editor()
            if not writer.setCell(cell):
                raise Exception, T.i18n("Invalid cell '%1' defined.", [cell])

            schedule = schedulewidget.currentSchedule()
            #print "schedule: ", schedule
            props = sourcewidget.selectedProperties()
            #print "props: ", props
            ot = sourcewidget.objectType()
            #print "objectType: ", ot
            return [ot, schedule, props, sourcewidget.includeHeaders() ]
        return None


PlanImport( self )
